# Generated by roxygen2: do not edit by hand

S3method(RunGGML,Seurat)
S3method(RunGGML,SingleCellExperiment)
S3method(RunGGML,default)
S3method(ag_eval,ag_batch_norm)
S3method(ag_eval,ag_dropout)
S3method(ag_eval,ag_embedding)
S3method(ag_eval,ag_layer_norm)
S3method(ag_eval,ag_multihead_attention)
S3method(ag_eval,ag_sequential)
S3method(ag_eval,default)
S3method(ag_train,ag_batch_norm)
S3method(ag_train,ag_dropout)
S3method(ag_train,ag_embedding)
S3method(ag_train,ag_layer_norm)
S3method(ag_train,ag_multihead_attention)
S3method(ag_train,ag_sequential)
S3method(ag_train,default)
S3method(as.matrix,ggml_matrix)
S3method(augment,ggmlr_parsnip_model)
S3method(compile,ggml_functional_model)
S3method(compile,ggml_sequential_model)
S3method(evaluate,ggml_functional_model)
S3method(evaluate,ggml_sequential_model)
S3method(fit,ggml_functional_model)
S3method(fit,ggml_sequential_model)
S3method(ggml_compile,ggml_functional_model)
S3method(ggml_compile,ggml_sequential_model)
S3method(ggml_evaluate,ggml_functional_model)
S3method(ggml_evaluate,ggml_sequential_model)
S3method(ggml_extract,Seurat)
S3method(ggml_extract,SingleCellExperiment)
S3method(ggml_extract,dgCMatrix)
S3method(ggml_extract,matrix)
S3method(ggml_fit,default)
S3method(ggml_fit,ggml_functional_model)
S3method(ggml_fit,ggml_sequential_model)
S3method(ggml_freeze_weights,ggml_functional_model)
S3method(ggml_freeze_weights,ggml_sequential_model)
S3method(ggml_inject,Seurat)
S3method(ggml_inject,SingleCellExperiment)
S3method(ggml_predict,ggml_functional_model)
S3method(ggml_predict,ggml_sequential_model)
S3method(ggml_save_model,ggml_functional_model)
S3method(ggml_save_model,ggml_sequential_model)
S3method(ggml_unfreeze_weights,ggml_functional_model)
S3method(ggml_unfreeze_weights,ggml_sequential_model)
S3method(glance,ggmlr_parsnip_model)
S3method(plot,ggml_history)
S3method(predict,ggml_functional_model)
S3method(predict,ggml_sequential_model)
S3method(predict,ggmlr_parsnip_model)
S3method(predict,onnx_model)
S3method(print,ag_dataloader)
S3method(print,ag_layer_norm)
S3method(print,ag_optimizer_adam)
S3method(print,ag_optimizer_sgd)
S3method(print,ag_sequential)
S3method(print,ag_tensor)
S3method(print,ggmlR_ag_state)
S3method(print,ggmlR_marshaled)
S3method(print,ggml_functional_model)
S3method(print,ggml_history)
S3method(print,ggml_result)
S3method(print,ggml_sequential_model)
S3method(print,ggml_task)
S3method(print,gguf)
S3method(print,lr_scheduler_cosine)
S3method(print,lr_scheduler_cyclic)
S3method(print,lr_scheduler_onecycle)
S3method(print,lr_scheduler_step)
S3method(print,lr_scheduler_warmup_cosine)
S3method(print,onnx_model)
S3method(print,pt_transformer)
S3method(summary,ggml_sequential_model)
S3method(tidy,ggmlr_parsnip_model)
export(GGML_GLU_OP_GEGLU)
export(GGML_GLU_OP_GEGLU_ERF)
export(GGML_GLU_OP_GEGLU_QUICK)
export(GGML_GLU_OP_REGLU)
export(GGML_GLU_OP_SWIGLU)
export(GGML_GLU_OP_SWIGLU_OAI)
export(GGML_N_TASKS_MAX)
export(GGML_OP_POOL_AVG)
export(GGML_OP_POOL_MAX)
export(GGML_ROPE_TYPE_MROPE)
export(GGML_ROPE_TYPE_NEOX)
export(GGML_ROPE_TYPE_NORM)
export(GGML_ROPE_TYPE_VISION)
export(GGML_SCALE_MODE_BICUBIC)
export(GGML_SCALE_MODE_BILINEAR)
export(GGML_SCALE_MODE_NEAREST)
export(GGML_SORT_ORDER_ASC)
export(GGML_SORT_ORDER_DESC)
export(GGML_TYPE_BF16)
export(GGML_TYPE_F16)
export(GGML_TYPE_F32)
export(GGML_TYPE_I32)
export(GGML_TYPE_Q2_K)
export(GGML_TYPE_Q3_K)
export(GGML_TYPE_Q4_0)
export(GGML_TYPE_Q4_1)
export(GGML_TYPE_Q4_K)
export(GGML_TYPE_Q5_K)
export(GGML_TYPE_Q6_K)
export(GGML_TYPE_Q8_0)
export(RunGGML)
export(ag_add)
export(ag_batch_norm)
export(ag_clamp)
export(ag_cross_entropy_loss)
export(ag_dataloader)
export(ag_default_device)
export(ag_default_dtype)
export(ag_device)
export(ag_dropout)
export(ag_dtype)
export(ag_embedding)
export(ag_eval)
export(ag_exp)
export(ag_gradcheck)
export(ag_layer_norm)
export(ag_linear)
export(ag_load_model)
export(ag_log)
export(ag_matmul)
export(ag_mean)
export(ag_mse_loss)
export(ag_mul)
export(ag_multihead_attention)
export(ag_param)
export(ag_pow)
export(ag_relu)
export(ag_reshape)
export(ag_save_model)
export(ag_scale)
export(ag_sequential)
export(ag_sigmoid)
export(ag_softmax)
export(ag_softmax_cross_entropy_loss)
export(ag_sub)
export(ag_sum)
export(ag_tanh)
export(ag_tensor)
export(ag_to_device)
export(ag_train)
export(ag_transpose)
export(as_gpu_matrix)
export(backward)
export(check_grad_anomaly)
export(clip_grad_norm)
export(clip_grad_value)
export(compile)
export(dequantize_row_iq1_m)
export(dequantize_row_iq1_s)
export(dequantize_row_iq2_s)
export(dequantize_row_iq2_xs)
export(dequantize_row_iq2_xxs)
export(dequantize_row_iq3_s)
export(dequantize_row_iq3_xxs)
export(dequantize_row_iq4_nl)
export(dequantize_row_iq4_xs)
export(dequantize_row_mxfp4)
export(dequantize_row_nvfp4)
export(dequantize_row_q1_0)
export(dequantize_row_q2_K)
export(dequantize_row_q3_K)
export(dequantize_row_q4_0)
export(dequantize_row_q4_1)
export(dequantize_row_q4_K)
export(dequantize_row_q5_0)
export(dequantize_row_q5_1)
export(dequantize_row_q5_K)
export(dequantize_row_q6_K)
export(dequantize_row_q8_0)
export(dequantize_row_q8_K)
export(dequantize_row_tq1_0)
export(dequantize_row_tq2_0)
export(dp_train)
export(evaluate)
export(fit)
export(ggml_abort_is_r_enabled)
export(ggml_abs)
export(ggml_abs_inplace)
export(ggml_add)
export(ggml_add1)
export(ggml_add_inplace)
export(ggml_add_rel_pos)
export(ggml_apply)
export(ggml_arange)
export(ggml_are_same_layout)
export(ggml_are_same_shape)
export(ggml_are_same_stride)
export(ggml_argmax)
export(ggml_argsort)
export(ggml_attention)
export(ggml_backend_alloc_ctx_tensors)
export(ggml_backend_buffer_clear)
export(ggml_backend_buffer_free)
export(ggml_backend_buffer_get_size)
export(ggml_backend_buffer_get_usage)
export(ggml_backend_buffer_is_host)
export(ggml_backend_buffer_is_multi_buffer)
export(ggml_backend_buffer_name)
export(ggml_backend_buffer_reset)
export(ggml_backend_buffer_set_usage)
export(ggml_backend_buffer_usage_any)
export(ggml_backend_buffer_usage_compute)
export(ggml_backend_buffer_usage_weights)
export(ggml_backend_buft_get_alignment)
export(ggml_backend_buft_get_max_size)
export(ggml_backend_buft_is_host)
export(ggml_backend_buft_name)
export(ggml_backend_cpu_init)
export(ggml_backend_cpu_set_n_threads)
export(ggml_backend_dev_buffer_type)
export(ggml_backend_dev_by_name)
export(ggml_backend_dev_by_type)
export(ggml_backend_dev_count)
export(ggml_backend_dev_description)
export(ggml_backend_dev_get)
export(ggml_backend_dev_get_props)
export(ggml_backend_dev_host_buffer_type)
export(ggml_backend_dev_init)
export(ggml_backend_dev_memory)
export(ggml_backend_dev_name)
export(ggml_backend_dev_offload_op)
export(ggml_backend_dev_supports_buft)
export(ggml_backend_dev_supports_op)
export(ggml_backend_dev_type)
export(ggml_backend_device_register)
export(ggml_backend_device_type_accel)
export(ggml_backend_device_type_cpu)
export(ggml_backend_device_type_gpu)
export(ggml_backend_device_type_igpu)
export(ggml_backend_event_free)
export(ggml_backend_event_new)
export(ggml_backend_event_record)
export(ggml_backend_event_synchronize)
export(ggml_backend_event_wait)
export(ggml_backend_free)
export(ggml_backend_get_device)
export(ggml_backend_graph_compute)
export(ggml_backend_graph_compute_async)
export(ggml_backend_graph_plan_compute)
export(ggml_backend_graph_plan_create)
export(ggml_backend_graph_plan_free)
export(ggml_backend_init_best)
export(ggml_backend_init_by_name)
export(ggml_backend_init_by_type)
export(ggml_backend_load)
export(ggml_backend_load_all)
export(ggml_backend_meta_device)
export(ggml_backend_multi_buffer_alloc_buffer)
export(ggml_backend_multi_buffer_set_usage)
export(ggml_backend_name)
export(ggml_backend_reg_by_name)
export(ggml_backend_reg_count)
export(ggml_backend_reg_dev_count)
export(ggml_backend_reg_dev_get)
export(ggml_backend_reg_get)
export(ggml_backend_reg_name)
export(ggml_backend_register)
export(ggml_backend_sched_alloc_graph)
export(ggml_backend_sched_free)
export(ggml_backend_sched_get_backend)
export(ggml_backend_sched_get_n_backends)
export(ggml_backend_sched_get_n_copies)
export(ggml_backend_sched_get_n_splits)
export(ggml_backend_sched_get_tensor_backend)
export(ggml_backend_sched_graph_compute)
export(ggml_backend_sched_graph_compute_async)
export(ggml_backend_sched_new)
export(ggml_backend_sched_reserve)
export(ggml_backend_sched_reset)
export(ggml_backend_sched_set_tensor_backend)
export(ggml_backend_sched_synchronize)
export(ggml_backend_sched_trace)
export(ggml_backend_synchronize)
export(ggml_backend_tensor_copy_async)
export(ggml_backend_tensor_get_and_sync)
export(ggml_backend_tensor_get_async)
export(ggml_backend_tensor_get_data)
export(ggml_backend_tensor_get_f32_first)
export(ggml_backend_tensor_set_async)
export(ggml_backend_tensor_set_data)
export(ggml_backend_unload)
export(ggml_batch_norm)
export(ggml_blck_size)
export(ggml_build_backward_expand)
export(ggml_build_forward_expand)
export(ggml_build_forward_expand_grads)
export(ggml_callback_early_stopping)
export(ggml_can_repeat)
export(ggml_cast)
export(ggml_cast_numeric)
export(ggml_ceil)
export(ggml_ceil_inplace)
export(ggml_clamp)
export(ggml_compile)
export(ggml_concat)
export(ggml_cont)
export(ggml_conv_1d)
export(ggml_conv_1d_dw)
export(ggml_conv_1d_layer)
export(ggml_conv_2d)
export(ggml_conv_2d_direct)
export(ggml_conv_2d_dw)
export(ggml_conv_2d_dw_direct)
export(ggml_conv_2d_layer)
export(ggml_conv_transpose_1d)
export(ggml_conv_transpose_2d_p0)
export(ggml_cos)
export(ggml_count_equal)
export(ggml_cpu_add)
export(ggml_cpu_features)
export(ggml_cpu_get_rvv_vlen)
export(ggml_cpu_get_sve_cnt)
export(ggml_cpu_has_amx_int8)
export(ggml_cpu_has_arm_fma)
export(ggml_cpu_has_avx)
export(ggml_cpu_has_avx2)
export(ggml_cpu_has_avx512)
export(ggml_cpu_has_avx512_bf16)
export(ggml_cpu_has_avx512_vbmi)
export(ggml_cpu_has_avx512_vnni)
export(ggml_cpu_has_avx_vnni)
export(ggml_cpu_has_bmi2)
export(ggml_cpu_has_dotprod)
export(ggml_cpu_has_f16c)
export(ggml_cpu_has_fma)
export(ggml_cpu_has_fp16_va)
export(ggml_cpu_has_llamafile)
export(ggml_cpu_has_matmul_int8)
export(ggml_cpu_has_neon)
export(ggml_cpu_has_riscv_v)
export(ggml_cpu_has_sme)
export(ggml_cpu_has_sse3)
export(ggml_cpu_has_ssse3)
export(ggml_cpu_has_sve)
export(ggml_cpu_has_vsx)
export(ggml_cpu_has_vxe)
export(ggml_cpu_has_wasm_simd)
export(ggml_cpu_mul)
export(ggml_cpy)
export(ggml_cross_entropy_loss)
export(ggml_cross_entropy_loss_back)
export(ggml_crossprod)
export(ggml_cumsum)
export(ggml_custom)
export(ggml_custom_inplace)
export(ggml_custom_ops)
export(ggml_cycles)
export(ggml_cycles_per_ms)
export(ggml_default_mlp)
export(ggml_dense)
export(ggml_diag)
export(ggml_diag_mask_inf)
export(ggml_diag_mask_inf_inplace)
export(ggml_diag_mask_zero)
export(ggml_div)
export(ggml_div_inplace)
export(ggml_dup)
export(ggml_dup_inplace)
export(ggml_dup_tensor)
export(ggml_element_size)
export(ggml_elu)
export(ggml_elu_inplace)
export(ggml_embedding)
export(ggml_estimate_memory)
export(ggml_evaluate)
export(ggml_exp)
export(ggml_exp_inplace)
export(ggml_extract)
export(ggml_fit)
export(ggml_fit_opt)
export(ggml_fit_opt_multi)
export(ggml_flash_attn_back)
export(ggml_flash_attn_ext)
export(ggml_flash_attn_ext_add_sinks)
export(ggml_flash_attn_ext_get_prec)
export(ggml_flash_attn_ext_set_prec)
export(ggml_floor)
export(ggml_floor_inplace)
export(ggml_free)
export(ggml_freeze_weights)
export(ggml_ftype_to_ggml_type)
export(ggml_gallocr_alloc_graph)
export(ggml_gallocr_free)
export(ggml_gallocr_get_buffer_size)
export(ggml_gallocr_new)
export(ggml_gallocr_new_buft)
export(ggml_gallocr_reserve)
export(ggml_gated_linear_attn)
export(ggml_gated_linear_attn_back)
export(ggml_geglu)
export(ggml_geglu_quick)
export(ggml_geglu_split)
export(ggml_gelu)
export(ggml_gelu_erf)
export(ggml_gelu_inplace)
export(ggml_gelu_quick)
export(ggml_get_f32)
export(ggml_get_f32_nd)
export(ggml_get_first_tensor)
export(ggml_get_i32)
export(ggml_get_i32_nd)
export(ggml_get_layer)
export(ggml_get_max_tensor_size)
export(ggml_get_mem_size)
export(ggml_get_n_threads)
export(ggml_get_name)
export(ggml_get_next_tensor)
export(ggml_get_no_alloc)
export(ggml_get_op_params)
export(ggml_get_op_params_f32)
export(ggml_get_op_params_i32)
export(ggml_get_rel_pos)
export(ggml_get_rows)
export(ggml_get_rows_back)
export(ggml_get_unary_op)
export(ggml_glu)
export(ggml_glu_split)
export(ggml_graph_add_node)
export(ggml_graph_clear)
export(ggml_graph_compute)
export(ggml_graph_compute_with_ctx)
export(ggml_graph_cpy)
export(ggml_graph_dump_dot)
export(ggml_graph_dup)
export(ggml_graph_expand)
export(ggml_graph_get_grad)
export(ggml_graph_get_grad_acc)
export(ggml_graph_get_tensor)
export(ggml_graph_n_nodes)
export(ggml_graph_node)
export(ggml_graph_overhead)
export(ggml_graph_print)
export(ggml_graph_reset)
export(ggml_graph_view)
export(ggml_group_norm)
export(ggml_group_norm_inplace)
export(ggml_gru)
export(ggml_hardsigmoid)
export(ggml_hardswish)
export(ggml_im2col)
export(ggml_init)
export(ggml_init_auto)
export(ggml_inject)
export(ggml_input)
export(ggml_is_available)
export(ggml_is_contiguous)
export(ggml_is_contiguous_0)
export(ggml_is_contiguous_1)
export(ggml_is_contiguous_2)
export(ggml_is_contiguous_channels)
export(ggml_is_contiguous_rows)
export(ggml_is_contiguously_allocated)
export(ggml_is_permuted)
export(ggml_is_quantized)
export(ggml_is_transposed)
export(ggml_l2_norm)
export(ggml_l2_norm_inplace)
export(ggml_layer_add)
export(ggml_layer_attention)
export(ggml_layer_batch_norm)
export(ggml_layer_concatenate)
export(ggml_layer_conv_1d)
export(ggml_layer_conv_2d)
export(ggml_layer_custom)
export(ggml_layer_dense)
export(ggml_layer_dropout)
export(ggml_layer_embedding)
export(ggml_layer_flatten)
export(ggml_layer_global_average_pooling_2d)
export(ggml_layer_global_max_pooling_2d)
export(ggml_layer_gru)
export(ggml_layer_lstm)
export(ggml_layer_max_pooling_2d)
export(ggml_leaky_relu)
export(ggml_load_model)
export(ggml_load_weights)
export(ggml_log)
export(ggml_log_inplace)
export(ggml_log_is_r_enabled)
export(ggml_log_set_default)
export(ggml_log_set_r)
export(ggml_lstm)
export(ggml_marshal_model)
export(ggml_matmul)
export(ggml_matmul_f64)
export(ggml_mean)
export(ggml_model)
export(ggml_model_backend)
export(ggml_model_sequential)
export(ggml_mul)
export(ggml_mul_inplace)
export(ggml_mul_mat)
export(ggml_mul_mat_id)
export(ggml_n_dims)
export(ggml_nbytes)
export(ggml_neg)
export(ggml_neg_inplace)
export(ggml_nelements)
export(ggml_new_f32)
export(ggml_new_i32)
export(ggml_new_tensor)
export(ggml_new_tensor_1d)
export(ggml_new_tensor_2d)
export(ggml_new_tensor_3d)
export(ggml_new_tensor_4d)
export(ggml_norm)
export(ggml_norm_inplace)
export(ggml_nrows)
export(ggml_op_can_inplace)
export(ggml_op_desc)
export(ggml_op_name)
export(ggml_op_symbol)
export(ggml_ops_registry)
export(ggml_opt_alloc)
export(ggml_opt_context_optimizer_type)
export(ggml_opt_dataset_data)
export(ggml_opt_dataset_free)
export(ggml_opt_dataset_get_batch)
export(ggml_opt_dataset_get_batch_head)
export(ggml_opt_dataset_init)
export(ggml_opt_dataset_labels)
export(ggml_opt_dataset_ndata)
export(ggml_opt_dataset_shuffle)
export(ggml_opt_dataset_weights)
export(ggml_opt_default_params)
export(ggml_opt_epoch)
export(ggml_opt_eval)
export(ggml_opt_fit)
export(ggml_opt_fit_multi)
export(ggml_opt_free)
export(ggml_opt_get_lr)
export(ggml_opt_grad_acc)
export(ggml_opt_init)
export(ggml_opt_init_for_fit)
export(ggml_opt_init_for_fit_multi)
export(ggml_opt_inputs)
export(ggml_opt_labels)
export(ggml_opt_labels_i)
export(ggml_opt_loss)
export(ggml_opt_loss_i)
export(ggml_opt_loss_type_binary_cross_entropy)
export(ggml_opt_loss_type_cross_entropy)
export(ggml_opt_loss_type_huber)
export(ggml_opt_loss_type_mae)
export(ggml_opt_loss_type_mean)
export(ggml_opt_loss_type_mse)
export(ggml_opt_loss_type_sum)
export(ggml_opt_loss_type_weighted_mse)
export(ggml_opt_loss_weights_i)
export(ggml_opt_n_loss)
export(ggml_opt_ncorrect)
export(ggml_opt_ncorrect_i)
export(ggml_opt_optimizer_name)
export(ggml_opt_optimizer_type_adamw)
export(ggml_opt_optimizer_type_sgd)
export(ggml_opt_outputs)
export(ggml_opt_outputs_i)
export(ggml_opt_pred)
export(ggml_opt_pred_i)
export(ggml_opt_prepare_alloc)
export(ggml_opt_reset)
export(ggml_opt_result_accuracy)
export(ggml_opt_result_accuracy_i)
export(ggml_opt_result_free)
export(ggml_opt_result_init)
export(ggml_opt_result_loss)
export(ggml_opt_result_loss_i)
export(ggml_opt_result_n_loss)
export(ggml_opt_result_ndata)
export(ggml_opt_result_pred)
export(ggml_opt_result_reset)
export(ggml_opt_set_lr)
export(ggml_opt_static_graphs)
export(ggml_out_prod)
export(ggml_pad)
export(ggml_pad_reflect_1d)
export(ggml_permute)
export(ggml_pool_1d)
export(ggml_pool_2d)
export(ggml_pop_layer)
export(ggml_pp_dp_forward)
export(ggml_pp_forward)
export(ggml_predict)
export(ggml_predict_classes)
export(ggml_print_mem_status)
export(ggml_print_objects)
export(ggml_quant_block_info)
export(ggml_quantize_chunk)
export(ggml_quantize_free)
export(ggml_quantize_init)
export(ggml_quantize_requires_imatrix)
export(ggml_reglu)
export(ggml_reglu_split)
export(ggml_relu)
export(ggml_relu_inplace)
export(ggml_repeat)
export(ggml_repeat_back)
export(ggml_reset)
export(ggml_reshape_1d)
export(ggml_reshape_2d)
export(ggml_reshape_3d)
export(ggml_reshape_4d)
export(ggml_result)
export(ggml_rms_norm)
export(ggml_rms_norm_back)
export(ggml_rms_norm_inplace)
export(ggml_roll)
export(ggml_rope)
export(ggml_rope_ext)
export(ggml_rope_ext_back)
export(ggml_rope_ext_inplace)
export(ggml_rope_inplace)
export(ggml_rope_multi)
export(ggml_rope_multi_back)
export(ggml_rope_multi_inplace)
export(ggml_round)
export(ggml_round_inplace)
export(ggml_run)
export(ggml_rwkv_output)
export(ggml_rwkv_state)
export(ggml_rwkv_wkv6)
export(ggml_rwkv_wkv6_back)
export(ggml_rwkv_wkv7)
export(ggml_rwkv_wkv7_back)
export(ggml_save_model)
export(ggml_save_weights)
export(ggml_scale)
export(ggml_scale_bias)
export(ggml_scale_inplace)
export(ggml_schedule_cosine_decay)
export(ggml_schedule_reduce_on_plateau)
export(ggml_schedule_step_decay)
export(ggml_set)
export(ggml_set_1d)
export(ggml_set_2d)
export(ggml_set_abort_callback_default)
export(ggml_set_abort_callback_r)
export(ggml_set_f32)
export(ggml_set_f32_nd)
export(ggml_set_i32)
export(ggml_set_i32_nd)
export(ggml_set_input)
export(ggml_set_loss)
export(ggml_set_n_threads)
export(ggml_set_name)
export(ggml_set_no_alloc)
export(ggml_set_omp_threads)
export(ggml_set_op_params)
export(ggml_set_op_params_f32)
export(ggml_set_op_params_i32)
export(ggml_set_output)
export(ggml_set_param)
export(ggml_set_seed)
export(ggml_set_zero)
export(ggml_sgn)
export(ggml_sigmoid)
export(ggml_sigmoid_inplace)
export(ggml_silu)
export(ggml_silu_back)
export(ggml_silu_inplace)
export(ggml_sin)
export(ggml_soft_max)
export(ggml_soft_max_add_sinks)
export(ggml_soft_max_ext)
export(ggml_soft_max_ext_back)
export(ggml_soft_max_ext_back_inplace)
export(ggml_soft_max_ext_inplace)
export(ggml_soft_max_inplace)
export(ggml_softplus)
export(ggml_softplus_inplace)
export(ggml_sqr)
export(ggml_sqr_inplace)
export(ggml_sqrt)
export(ggml_sqrt_inplace)
export(ggml_ssm_conv)
export(ggml_ssm_conv_back)
export(ggml_ssm_scan)
export(ggml_ssm_scan_back)
export(ggml_ssm_scan_output)
export(ggml_ssm_scan_state)
export(ggml_step)
export(ggml_sub)
export(ggml_sub_inplace)
export(ggml_sum)
export(ggml_sum_rows)
export(ggml_swiglu)
export(ggml_swiglu_split)
export(ggml_tanh)
export(ggml_tanh_inplace)
export(ggml_task)
export(ggml_tcrossprod)
export(ggml_tensor_copy)
export(ggml_tensor_data_ptr)
export(ggml_tensor_nb)
export(ggml_tensor_num)
export(ggml_tensor_overhead)
export(ggml_tensor_set_f32_scalar)
export(ggml_tensor_shape)
export(ggml_tensor_type)
export(ggml_test)
export(ggml_test_adamw_steps)
export(ggml_test_backend_ops)
export(ggml_time_init)
export(ggml_time_ms)
export(ggml_time_us)
export(ggml_timestep_embedding)
export(ggml_top_k)
export(ggml_tp_dp_forward)
export(ggml_training_history)
export(ggml_transpose)
export(ggml_type_name)
export(ggml_type_size)
export(ggml_type_sizef)
export(ggml_unary_op_name)
export(ggml_unfreeze_weights)
export(ggml_unmarshal_model)
export(ggml_upscale)
export(ggml_used_mem)
export(ggml_version)
export(ggml_view_1d)
export(ggml_view_2d)
export(ggml_view_3d)
export(ggml_view_4d)
export(ggml_view_tensor)
export(ggml_vulkan_available)
export(ggml_vulkan_backend_name)
export(ggml_vulkan_device_caps)
export(ggml_vulkan_device_count)
export(ggml_vulkan_device_description)
export(ggml_vulkan_device_groups)
export(ggml_vulkan_device_memory)
export(ggml_vulkan_free)
export(ggml_vulkan_hard_exit_available)
export(ggml_vulkan_init)
export(ggml_vulkan_is_backend)
export(ggml_vulkan_list_devices)
export(ggml_vulkan_p2p_selftest)
export(ggml_vulkan_shutdown)
export(ggml_vulkan_split_buffer_type)
export(ggml_vulkan_split_mul_mat)
export(ggml_vulkan_split_row_ranges)
export(ggml_vulkan_status)
export(ggml_win_part)
export(ggml_win_unpart)
export(ggml_with_temp_ctx)
export(ggmlr_parsnip_fit_classif)
export(ggmlr_parsnip_fit_regr)
export(gguf_free)
export(gguf_load)
export(gguf_metadata)
export(gguf_tensor_data)
export(gguf_tensor_info)
export(gguf_tensor_names)
export(iq2xs_free_impl)
export(iq2xs_init_impl)
export(iq3xs_free_impl)
export(iq3xs_init_impl)
export(lr_scheduler_cosine)
export(lr_scheduler_cyclic)
export(lr_scheduler_onecycle)
export(lr_scheduler_step)
export(lr_scheduler_warmup_cosine)
export(nn_topo_sort)
export(onnx_device_info)
export(onnx_inputs)
export(onnx_load)
export(onnx_run)
export(onnx_summary)
export(optimizer_adam)
export(optimizer_sgd)
export(pt_cube4_layout)
export(pt_forward)
export(pt_transformer)
export(pt_transformer_load)
export(pth_catalogue)
export(pth_load)
export(quantize_iq1_m)
export(quantize_iq1_s)
export(quantize_iq2_s)
export(quantize_iq2_xs)
export(quantize_iq2_xxs)
export(quantize_iq3_s)
export(quantize_iq3_xxs)
export(quantize_iq4_nl)
export(quantize_iq4_xs)
export(quantize_mxfp4)
export(quantize_nvfp4)
export(quantize_q1_0)
export(quantize_q2_K)
export(quantize_q3_K)
export(quantize_q4_0)
export(quantize_q4_1)
export(quantize_q4_K)
export(quantize_q5_0)
export(quantize_q5_1)
export(quantize_q5_K)
export(quantize_q6_K)
export(quantize_q8_0)
export(quantize_row_iq2_s_ref)
export(quantize_row_iq3_s_ref)
export(quantize_row_iq3_xxs_ref)
export(quantize_row_iq4_nl_ref)
export(quantize_row_iq4_xs_ref)
export(quantize_row_mxfp4_ref)
export(quantize_row_nvfp4_ref)
export(quantize_row_q1_0_ref)
export(quantize_row_q2_K_ref)
export(quantize_row_q3_K_ref)
export(quantize_row_q4_0_ref)
export(quantize_row_q4_1_ref)
export(quantize_row_q4_K_ref)
export(quantize_row_q5_0_ref)
export(quantize_row_q5_1_ref)
export(quantize_row_q5_K_ref)
export(quantize_row_q6_K_ref)
export(quantize_row_q8_0_ref)
export(quantize_row_q8_1_ref)
export(quantize_row_q8_K_ref)
export(quantize_row_tq1_0_ref)
export(quantize_row_tq2_0_ref)
export(quantize_tq1_0)
export(quantize_tq2_0)
export(with_grad_tape)
exportClasses(ggml_matrix)
exportMethods("%*%")
exportMethods(crossprod)
exportMethods(dim)
exportMethods(show)
exportMethods(tcrossprod)
importFrom(generics,augment)
importFrom(generics,compile)
importFrom(generics,fit)
importFrom(generics,glance)
importFrom(generics,tidy)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(methods,as)
importFrom(methods,new)
importFrom(methods,setGeneric)
importFrom(methods,show)
importFrom(stats,predict)
importFrom(stats,runif)
useDynLib(ggmlR, .registration = TRUE)
