% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_clamp}
\alias{ggml_clamp}
\title{Clamp (Graph)}
\usage{
ggml_clamp(ctx, a, min_val, max_val)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Input tensor}

\item{min_val}{Minimum value}

\item{max_val}{Maximum value}
}
\value{
Tensor with values clamped to [min_val, max_val]
}
\description{
Creates a graph node for clamping values to a range: clamp(x, min, max)
}
