% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/memory.R
\name{ggml_print_mem_status}
\alias{ggml_print_mem_status}
\title{Print Context Memory Status}
\usage{
ggml_print_mem_status(ctx)
}
\arguments{
\item{ctx}{GGML context}
}
\value{
List with total, used, free memory (invisible)
}
\description{
Helper to print memory usage information
}
\examples{
\donttest{
ctx <- ggml_init(1024 * 1024)
ggml_print_mem_status(ctx)
ggml_free(ctx)
}
}
