% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggml.R
\name{ggml_version}
\alias{ggml_version}
\title{Get GGML version}
\usage{
ggml_version()
}
\value{
Character string with GGML version
}
\description{
Get GGML version
}
\examples{
\donttest{
ggml_version()
}
}
