% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulkan.R
\name{ggml_vulkan_device_caps}
\alias{ggml_vulkan_device_caps}
\title{Get Vulkan device capabilities}
\usage{
ggml_vulkan_device_caps(device = 0L)
}
\arguments{
\item{device}{Device index (0-based, default 0)}
}
\value{
Named list: coopmat_support, coopmat1_fa_support, fp16, subgroup_size, subgroup_no_shmem
}
\description{
Returns hardware capabilities for the specified Vulkan device.
}
