% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_backend_tensor_get_f32_first}
\alias{ggml_backend_tensor_get_f32_first}
\title{Get First Float from Backend Tensor}
\usage{
ggml_backend_tensor_get_f32_first(tensor)
}
\arguments{
\item{tensor}{Tensor pointer}
}
\value{
Float value
}
\description{
Reads the first f32 element from a backend tensor.
}
