% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_backend_tensor_get_and_sync}
\alias{ggml_backend_tensor_get_and_sync}
\title{Backend Tensor Get and Sync}
\usage{
ggml_backend_tensor_get_and_sync(backend, tensor, offset = 0, size)
}
\arguments{
\item{backend}{Backend pointer (or NULL for CPU)}

\item{tensor}{Tensor pointer}

\item{offset}{Byte offset (default 0)}

\item{size}{Number of bytes to read}
}
\value{
Raw vector with tensor data
}
\description{
Gets tensor data from a backend with synchronization.
}
