% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scheduler.R
\name{ggml_backend_sched_alloc_graph}
\alias{ggml_backend_sched_alloc_graph}
\title{Allocate graph on scheduler}
\usage{
ggml_backend_sched_alloc_graph(sched, graph)
}
\arguments{
\item{sched}{Scheduler pointer}

\item{graph}{Graph pointer}
}
\value{
Logical indicating success
}
\description{
Allocates memory for a graph across the scheduler's backends.
Must be called before computing the graph.
}
