% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/memory.R
\name{ggml_get_mem_size}
\alias{ggml_get_mem_size}
\title{Get Context Memory Size}
\usage{
ggml_get_mem_size(ctx)
}
\arguments{
\item{ctx}{GGML context}
}
\value{
Total memory size in bytes
}
\description{
Returns the total memory pool size of the context
}
\examples{
\donttest{
ctx <- ggml_init(1024 * 1024)
ggml_get_mem_size(ctx)
ggml_free(ctx)
}
}
