% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scheduler.R
\name{ggml_backend_sched_get_tensor_backend}
\alias{ggml_backend_sched_get_tensor_backend}
\title{Get tensor backend assignment}
\usage{
ggml_backend_sched_get_tensor_backend(sched, tensor)
}
\arguments{
\item{sched}{Scheduler pointer}

\item{tensor}{Tensor pointer}
}
\value{
Backend pointer or NULL if not assigned
}
\description{
Returns which backend a tensor is assigned to.
}
