% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nn_model.R
\name{summary.ggml_sequential_model}
\alias{summary.ggml_sequential_model}
\title{Summary method for ggml_sequential_model}
\usage{
\method{summary}{ggml_sequential_model}(object, ...)
}
\arguments{
\item{object}{A ggml_sequential_model object}

\item{...}{Additional arguments (ignored)}
}
\value{
The model object (invisibly).
}
\description{
Prints a detailed summary including input shape, layer details,
trainable/non-trainable parameter counts, and memory estimate.
}
