% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_is_contiguous_1}
\alias{ggml_is_contiguous_1}
\title{Check Tensor Contiguity (Dimensions >= 1)}
\usage{
ggml_is_contiguous_1(tensor)
}
\arguments{
\item{tensor}{Tensor pointer}
}
\value{
Logical indicating contiguity for dims >= 1
}
\description{
Check if tensor is contiguous for dimensions >= 1.
Allows non-contiguous first dimension.
}
\seealso{
Other tensor_layout: 
\code{\link{ggml_are_same_stride}()},
\code{\link{ggml_can_repeat}()},
\code{\link{ggml_count_equal}()},
\code{\link{ggml_is_contiguous_0}()},
\code{\link{ggml_is_contiguous_2}()},
\code{\link{ggml_is_contiguous_channels}()},
\code{\link{ggml_is_contiguous_rows}()},
\code{\link{ggml_is_contiguously_allocated}()}
}
\concept{tensor_layout}
