% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scheduler.R
\name{ggml_backend_sched_get_backend}
\alias{ggml_backend_sched_get_backend}
\title{Get backend from scheduler}
\usage{
ggml_backend_sched_get_backend(sched, index = 0L)
}
\arguments{
\item{sched}{Scheduler pointer}

\item{index}{Backend index (0-based)}
}
\value{
Backend pointer
}
\description{
Returns a specific backend from the scheduler by index.
}
