% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nn_functional.R
\name{ggml_gru}
\alias{ggml_gru}
\title{Create a GRU Layer Object}
\usage{
ggml_gru(
  units,
  return_sequences = FALSE,
  activation = "tanh",
  recurrent_activation = "sigmoid",
  name = NULL,
  trainable = TRUE
)
}
\arguments{
\item{units}{Integer, number of hidden units.}

\item{return_sequences}{Logical.}

\item{activation}{Candidate activation (default \code{"tanh"}).}

\item{recurrent_activation}{Gate activation (default \code{"sigmoid"}).}

\item{name}{Optional character name.}

\item{trainable}{Logical.}
}
\value{
A \code{ggml_layer} object.
}
\description{
Create a GRU Layer Object
}
