% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph.R
\name{ggml_backend_alloc_ctx_tensors}
\alias{ggml_backend_alloc_ctx_tensors}
\title{Allocate Context Tensors to Backend}
\usage{
ggml_backend_alloc_ctx_tensors(ctx, backend)
}
\arguments{
\item{ctx}{GGML context}

\item{backend}{Backend handle}
}
\value{
Backend buffer object
}
\description{
Allocates all tensors in a GGML context to a specific backend.
Returns a buffer that must be freed when no longer needed.
}
