% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quants.R
\name{iq3xs_init_impl}
\alias{iq3xs_init_impl}
\title{Initialize IQ3 Quantization Tables}
\usage{
iq3xs_init_impl(grid_size)
}
\arguments{
\item{grid_size}{Grid size for IQ3 (typically 256)}
}
\value{
NULL invisibly
}
\description{
Initializes lookup tables for IQ3 quantization types.
Must be called before using iq3_xxs or iq3_s quantization.
}
\seealso{
Other quantization: 
\code{\link{dequantize_row_iq2_xxs}()},
\code{\link{dequantize_row_mxfp4}()},
\code{\link{dequantize_row_q2_K}()},
\code{\link{dequantize_row_q4_0}()},
\code{\link{dequantize_row_tq1_0}()},
\code{\link{ggml_quant_block_info}()},
\code{\link{iq2xs_free_impl}()},
\code{\link{iq2xs_init_impl}()},
\code{\link{iq3xs_free_impl}()},
\code{\link{quantize_iq2_xxs}()},
\code{\link{quantize_mxfp4}()},
\code{\link{quantize_q2_K}()},
\code{\link{quantize_q4_0}()},
\code{\link{quantize_row_iq3_xxs_ref}()},
\code{\link{quantize_row_mxfp4_ref}()},
\code{\link{quantize_row_q2_K_ref}()},
\code{\link{quantize_row_q4_0_ref}()},
\code{\link{quantize_row_tq1_0_ref}()},
\code{\link{quantize_tq1_0}()}
}
\concept{quantization}
