% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_conv_2d_direct}
\alias{ggml_conv_2d_direct}
\title{Direct 2D Convolution (Graph)}
\usage{
ggml_conv_2d_direct(
  ctx,
  a,
  b,
  s0 = 1L,
  s1 = 1L,
  p0 = 0L,
  p1 = 0L,
  d0 = 1L,
  d1 = 1L
)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Convolution kernel tensor}

\item{b}{Input data tensor}

\item{s0}{Stride dimension 0 (default 1)}

\item{s1}{Stride dimension 1 (default 1)}

\item{p0}{Padding dimension 0 (default 0)}

\item{p1}{Padding dimension 1 (default 0)}

\item{d0}{Dilation dimension 0 (default 1)}

\item{d1}{Dilation dimension 1 (default 1)}
}
\value{
Convolved tensor
}
\description{
Applies 2D convolution using the direct algorithm (no im2col).
}
