% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_quantize_requires_imatrix}
\alias{ggml_quantize_requires_imatrix}
\title{Check if Quantization Requires Importance Matrix}
\usage{
ggml_quantize_requires_imatrix(type)
}
\arguments{
\item{type}{GGML type}
}
\value{
TRUE if importance matrix is required
}
\description{
Some quantization types require an importance matrix for optimal quality.
}
