% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nn_model.R
\name{plot.ggml_history}
\alias{plot.ggml_history}
\title{Plot training history}
\usage{
\method{plot}{ggml_history}(x, ...)
}
\arguments{
\item{x}{A ggml_history object}

\item{...}{Additional arguments (ignored)}
}
\value{
The history object (invisibly).
}
\description{
Plots loss and accuracy curves over epochs.
}
