% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autograd.R
\name{is_ag_tensor}
\alias{is_ag_tensor}
\title{Check if object is an ag_tensor}
\usage{
is_ag_tensor(x)
}
\description{
Check if object is an ag_tensor
}
\keyword{internal}
