% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nn_functional.R
\name{nn_build_functional_node}
\alias{nn_build_functional_node}
\title{Build a single ggml tensor for one functional node}
\usage{
nn_build_functional_node(
  node,
  built_tensors,
  built_shapes,
  ctx_weights,
  ctx_compute,
  batch_size,
  training = FALSE,
  reuse_weights = NULL
)
}
\arguments{
\item{reuse_weights}{Named list of pre-allocated weight tensors to reuse
(for shared layers -- second+ application of a named layer).  When not
NULL the function uses these tensors instead of allocating new ones.}
}
\description{
Build a single ggml tensor for one functional node
}
\keyword{internal}
