% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_set_2d}
\alias{ggml_set_2d}
\title{Set 2D Tensor Region (Graph)}
\usage{
ggml_set_2d(ctx, a, b, nb1, offset)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Destination tensor}

\item{b}{Source tensor}

\item{nb1}{Stride for dimension 1 (in bytes)}

\item{offset}{Byte offset in destination tensor}
}
\value{
Tensor representing the set operation
}
\description{
Simplified 2D version of ggml_set.
}
