% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_is_quantized}
\alias{ggml_is_quantized}
\title{Check If Type is Quantized}
\usage{
ggml_is_quantized(type)
}
\arguments{
\item{type}{GGML type constant}
}
\value{
Logical indicating if type is quantized
}
\description{
Returns TRUE if the GGML type is a quantized format.
}
\examples{
ggml_is_quantized(GGML_TYPE_F32)  # FALSE
ggml_is_quantized(GGML_TYPE_Q4_0) # TRUE
}
\seealso{
Other type_system: 
\code{\link{ggml_blck_size}()},
\code{\link{ggml_ftype_to_ggml_type}()},
\code{\link{ggml_type_name}()},
\code{\link{ggml_type_sizef}()}
}
\concept{type_system}
