% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_conv_1d}
\alias{ggml_conv_1d}
\title{1D Convolution (Graph)}
\usage{
ggml_conv_1d(ctx, a, b, s0 = 1L, p0 = 0L, d0 = 1L)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Convolution kernel tensor}

\item{b}{Input data tensor}

\item{s0}{Stride (default 1)}

\item{p0}{Padding (default 0)}

\item{d0}{Dilation (default 1)}
}
\value{
Convolved tensor
}
\description{
Applies 1D convolution to input data.
}
