% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/threading.R
\name{ggml_set_omp_threads}
\alias{ggml_set_omp_threads}
\title{Set OpenMP Thread Count}
\usage{
ggml_set_omp_threads(n)
}
\arguments{
\item{n}{Number of threads}
}
\value{
NULL invisibly
}
\description{
Directly calls omp_set_num_threads() to limit OpenMP parallelism.
Useful in tests to comply with CRAN policy on core usage.
}
