% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scheduler.R
\name{ggml_backend_sched_set_tensor_backend}
\alias{ggml_backend_sched_set_tensor_backend}
\title{Set tensor backend assignment}
\usage{
ggml_backend_sched_set_tensor_backend(sched, tensor, backend)
}
\arguments{
\item{sched}{Scheduler pointer}

\item{tensor}{Tensor pointer}

\item{backend}{Backend pointer to assign tensor to}
}
\value{
NULL (invisible)
}
\description{
Manually assigns a specific tensor to run on a specific backend.
This overrides automatic scheduling.
}
