% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_tensor_set_f32_scalar}
\alias{ggml_tensor_set_f32_scalar}
\title{Fill Tensor with Scalar}
\usage{
ggml_tensor_set_f32_scalar(tensor, value)
}
\arguments{
\item{tensor}{Tensor pointer}

\item{value}{Float value to fill with}
}
\value{
NULL (invisible)
}
\description{
Sets all elements of a f32 tensor to a single value.
}
