% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_geglu_quick}
\alias{ggml_geglu_quick}
\title{GeGLU Quick (Fast GeGLU) (Graph)}
\usage{
ggml_geglu_quick(ctx, a)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Input tensor (first dimension must be even)}
}
\value{
Tensor with half the first dimension of input
}
\description{
Creates a graph node for fast GeGLU approximation.
Uses faster but less accurate GELU approximation for gating.
}
