% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulkan.R
\name{ggml_vulkan_free}
\alias{ggml_vulkan_free}
\title{Free Vulkan backend}
\usage{
ggml_vulkan_free(backend)
}
\arguments{
\item{backend}{Vulkan backend pointer from ggml_vulkan_init()}
}
\value{
NULL (invisible)
}
\description{
Releases resources associated with the Vulkan backend.
}
\examples{
\donttest{
if (ggml_vulkan_available() && ggml_vulkan_device_count() > 0) {
  backend <- ggml_vulkan_init(0)
  ggml_vulkan_free(backend)
}
}
}
