% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggml.R
\name{ggml_free}
\alias{ggml_free}
\title{Free GGML context}
\usage{
ggml_free(ctx)
}
\arguments{
\item{ctx}{Context pointer}
}
\value{
NULL (invisible)
}
\description{
Free GGML context
}
\examples{
\donttest{
ctx <- ggml_init(1024 * 1024)
ggml_free(ctx)
}
}
