% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_op_name}
\alias{ggml_op_name}
\title{Get Operation Name}
\usage{
ggml_op_name(op)
}
\arguments{
\item{op}{GGML operation constant}
}
\value{
Character string with operation name
}
\description{
Returns the string name of a GGML operation.
}
\seealso{
Other op_info: 
\code{\link{ggml_get_unary_op}()},
\code{\link{ggml_op_desc}()},
\code{\link{ggml_op_symbol}()},
\code{\link{ggml_unary_op_name}()}
}
\concept{op_info}
