% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggml.R
\name{ggml_time_ms}
\alias{ggml_time_ms}
\title{Get Time in Milliseconds}
\usage{
ggml_time_ms()
}
\value{
Numeric value representing milliseconds
}
\description{
Returns the current time in milliseconds since the timer was initialized.
}
\examples{
\donttest{
ggml_time_init()
start <- ggml_time_ms()
Sys.sleep(0.01)
elapsed <- ggml_time_ms() - start
}
}
