% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/onnx.R
\name{onnx_summary}
\alias{onnx_summary}
\title{ONNX model summary}
\usage{
onnx_summary(model)
}
\arguments{
\item{model}{An \code{onnx_model} object from \code{onnx_load()}.}
}
\value{
A list with \code{ir_version}, \code{opset_version},
  \code{producer}, \code{graph_name}, \code{n_nodes},
  \code{n_initializers}, and \code{ops}.
}
\description{
Returns metadata about a loaded ONNX model.
}
