% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggml.R
\name{ggml_cycles_per_ms}
\alias{ggml_cycles_per_ms}
\title{Get CPU Cycles per Millisecond}
\usage{
ggml_cycles_per_ms()
}
\value{
Numeric value representing cycles per millisecond
}
\description{
Returns an estimate of CPU cycles per millisecond.
Useful for converting cycle counts to time.
}
\examples{
\donttest{
ggml_cycles_per_ms()
}
}
