% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scheduler.R
\name{ggml_backend_sched_free}
\alias{ggml_backend_sched_free}
\title{Free backend scheduler}
\usage{
ggml_backend_sched_free(sched)
}
\arguments{
\item{sched}{Scheduler pointer from ggml_backend_sched_new()}
}
\value{
NULL (invisible)
}
\description{
Releases resources associated with the backend scheduler.
}
\examples{
\donttest{
cpu <- ggml_backend_cpu_init()
sched <- ggml_backend_sched_new(list(cpu))
ggml_backend_sched_free(sched)
ggml_backend_free(cpu)
}
}
