% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autograd.R
\name{ag_matmul}
\alias{ag_matmul}
\title{Matrix multiplication}
\usage{
ag_matmul(A, B)
}
\arguments{
\item{A}{ag_tensor or numeric matrix of shape \code{[m, k]}}

\item{B}{ag_tensor or numeric matrix of shape \code{[k, n]}}
}
\value{
ag_tensor of shape \code{[m, n]}
}
\description{
Computes \code{A \%*\% B} and records the operation on the gradient tape.
}
