% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_backend_name}
\alias{ggml_backend_name}
\title{Get Backend Name}
\usage{
ggml_backend_name(backend)
}
\arguments{
\item{backend}{Backend pointer}
}
\value{
Character string name
}
\description{
Returns the name of the backend (e.g., "CPU").
}
