% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_backend_cpu_init}
\alias{ggml_backend_cpu_init}
\title{Initialize CPU Backend}
\usage{
ggml_backend_cpu_init()
}
\value{
Backend pointer
}
\description{
Creates a new CPU backend instance for graph computation.
}
