% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nn_layers.R
\name{nn_build_global_max_pooling_2d}
\alias{nn_build_global_max_pooling_2d}
\title{Build global_max_pooling_2d forward pass}
\usage{
nn_build_global_max_pooling_2d(ctx, input_tensor, layer)
}
\description{
Tensor layout in ggml (column-major): [W, H, C, N].
Pool the entire spatial extent (k0=W, k1=H), then reshape [1,1,C,N]->[C,N].
}
\keyword{internal}
