% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nn_layers.R
\name{nn_apply_activation}
\alias{nn_apply_activation}
\title{Apply activation function}
\usage{
nn_apply_activation(ctx, tensor, activation)
}
\description{
Apply activation function
}
\keyword{internal}
