% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_reglu_split}
\alias{ggml_reglu_split}
\title{ReGLU Split (Graph)}
\usage{
ggml_reglu_split(ctx, a, b)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Input tensor (the values to be gated)}

\item{b}{Gate tensor (same shape as a)}
}
\value{
Tensor with same shape as input tensors
}
\description{
Creates a graph node for ReGLU with separate input and gate tensors.
}
\details{
Formula: output = ReLU(a) * b
}
