% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggml.R
\name{ggml_time_us}
\alias{ggml_time_us}
\title{Get Time in Microseconds}
\usage{
ggml_time_us()
}
\value{
Numeric value representing microseconds
}
\description{
Returns the current time in microseconds since the timer was initialized.
More precise than ggml_time_ms() for micro-benchmarking.
}
\examples{
\donttest{
ggml_time_init()
start <- ggml_time_us()
Sys.sleep(0.001)
elapsed <- ggml_time_us() - start
}
}
