% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulkan.R
\name{ggml_vulkan_available}
\alias{ggml_vulkan_available}
\title{Check if Vulkan support is available}
\usage{
ggml_vulkan_available()
}
\value{
Logical indicating if Vulkan is available
}
\description{
Returns TRUE if the package was compiled with Vulkan support.
To enable Vulkan, install libvulkan-dev and glslc, then reinstall ggmlR.
}
\examples{
ggml_vulkan_available()
}
