% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_set_name}
\alias{ggml_set_name}
\title{Set Tensor Name}
\usage{
ggml_set_name(tensor, name)
}
\arguments{
\item{tensor}{Tensor pointer}

\item{name}{Character string name}
}
\value{
The tensor (for chaining)
}
\description{
Assigns a name to a tensor. Useful for debugging and graph visualization.
}
