% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/onnx.R
\name{onnx_inputs}
\alias{onnx_inputs}
\title{List ONNX model inputs}
\usage{
onnx_inputs(model)
}
\arguments{
\item{model}{An \code{onnx_model} object from \code{onnx_load()}.}
}
\value{
A named list where names are input tensor names and values
  are integer vectors of dimension sizes (-1 for dynamic dimensions).
}
\description{
Returns the names and shapes of model inputs (excluding weight
initializers). Use this to know what to pass to \code{onnx_run()}.
}
