% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_set_1d}
\alias{ggml_set_1d}
\title{Set 1D Tensor Region (Graph)}
\usage{
ggml_set_1d(ctx, a, b, offset)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Destination tensor}

\item{b}{Source tensor}

\item{offset}{Byte offset in destination tensor}
}
\value{
Tensor representing the set operation
}
\description{
Simplified 1D version of ggml_set.
Copies tensor b into tensor a starting at offset.
}
