% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggml.R
\name{ggml_test}
\alias{ggml_test}
\title{Test GGML}
\usage{
ggml_test()
}
\value{
TRUE if test passed
}
\description{
Runs GGML library self-test and prints version info.
}
\examples{
\donttest{
ggml_test()
}
}
