% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_cpu_get_sve_cnt}
\alias{ggml_cpu_get_sve_cnt}
\title{Get SVE Vector Length (ARM)}
\usage{
ggml_cpu_get_sve_cnt()
}
\value{
Integer vector length in bytes
}
\description{
Returns the SVE vector length in bytes (0 if not supported).
}
\seealso{
Other cpu_features: 
\code{\link{ggml_cpu_features}()},
\code{\link{ggml_cpu_get_rvv_vlen}()},
\code{\link{ggml_cpu_has_amx_int8}()},
\code{\link{ggml_cpu_has_arm_fma}()},
\code{\link{ggml_cpu_has_avx}()},
\code{\link{ggml_cpu_has_avx2}()},
\code{\link{ggml_cpu_has_avx512}()},
\code{\link{ggml_cpu_has_avx512_bf16}()},
\code{\link{ggml_cpu_has_avx512_vbmi}()},
\code{\link{ggml_cpu_has_avx512_vnni}()},
\code{\link{ggml_cpu_has_avx_vnni}()},
\code{\link{ggml_cpu_has_bmi2}()},
\code{\link{ggml_cpu_has_dotprod}()},
\code{\link{ggml_cpu_has_f16c}()},
\code{\link{ggml_cpu_has_fma}()},
\code{\link{ggml_cpu_has_fp16_va}()},
\code{\link{ggml_cpu_has_llamafile}()},
\code{\link{ggml_cpu_has_matmul_int8}()},
\code{\link{ggml_cpu_has_neon}()},
\code{\link{ggml_cpu_has_riscv_v}()},
\code{\link{ggml_cpu_has_sme}()},
\code{\link{ggml_cpu_has_sse3}()},
\code{\link{ggml_cpu_has_ssse3}()},
\code{\link{ggml_cpu_has_sve}()},
\code{\link{ggml_cpu_has_vsx}()},
\code{\link{ggml_cpu_has_vxe}()},
\code{\link{ggml_cpu_has_wasm_simd}()}
}
\concept{cpu_features}
