% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph.R
\name{ggml_gallocr_get_buffer_size}
\alias{ggml_gallocr_get_buffer_size}
\title{Get Graph Allocator Buffer Size}
\usage{
ggml_gallocr_get_buffer_size(galloc, buffer_id = 0L)
}
\arguments{
\item{galloc}{Graph allocator object}

\item{buffer_id}{Buffer ID (default: 0 for single-buffer allocator)}
}
\value{
Size in bytes
}
\description{
Returns the size of the buffer used by the graph allocator.
}
