% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggml.R
\name{ggml_init}
\alias{ggml_init}
\title{Initialize GGML context}
\usage{
ggml_init(mem_size = 16 * 1024 * 1024, no_alloc = FALSE)
}
\arguments{
\item{mem_size}{Memory size in bytes}

\item{no_alloc}{If TRUE, don't allocate memory for tensors (default: FALSE)}
}
\value{
GGML context pointer
}
\description{
Initialize GGML context
}
\examples{
\donttest{
ctx <- ggml_init(1024 * 1024)
ggml_free(ctx)
}
}
