% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gguf.R
\name{gguf_tensor_data}
\alias{gguf_tensor_data}
\title{Extract Tensor Data}
\usage{
gguf_tensor_data(x, name)
}
\arguments{
\item{x}{A \code{gguf} object.}

\item{name}{Tensor name (character).}
}
\value{
A numeric array.
}
\description{
Dequantizes (if needed) and returns tensor weights as an R numeric array
with dimensions matching the tensor shape.
}
