% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autograd.R
\name{ag_sigmoid}
\alias{ag_sigmoid}
\title{Sigmoid activation}
\usage{
ag_sigmoid(x)
}
\arguments{
\item{x}{ag_tensor}
}
\value{
ag_tensor
}
\description{
Applies \eqn{1 / (1 + e^{-x})}.
}
