% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_backend_graph_compute}
\alias{ggml_backend_graph_compute}
\title{Compute Graph with Backend}
\usage{
ggml_backend_graph_compute(backend, graph)
}
\arguments{
\item{backend}{Backend pointer}

\item{graph}{Graph pointer}
}
\value{
Status code (0 = success)
}
\description{
Executes computation graph using specified backend.
}
