% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_soft_max_ext_back}
\alias{ggml_soft_max_ext_back}
\title{Softmax Backward Extended (Graph)}
\usage{
ggml_soft_max_ext_back(ctx, a, b, scale = 1, max_bias = 0)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Softmax output tensor (from forward pass)}

\item{b}{Gradient tensor from upstream}

\item{scale}{Scale factor (same as forward pass)}

\item{max_bias}{Maximum ALiBi bias (same as forward pass)}
}
\value{
Gradient tensor for the input
}
\description{
Backward pass for extended softmax operation.
}
