% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scheduler.R
\name{ggml_backend_sched_graph_compute_async}
\alias{ggml_backend_sched_graph_compute_async}
\title{Compute graph asynchronously}
\usage{
ggml_backend_sched_graph_compute_async(sched, graph)
}
\arguments{
\item{sched}{Scheduler pointer}

\item{graph}{Graph pointer}
}
\value{
Status code (0 = success)
}
\description{
Computes a graph asynchronously across backends.
Use ggml_backend_sched_synchronize() to wait for completion.
}
