% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gguf.R
\name{gguf_metadata}
\alias{gguf_metadata}
\title{Get GGUF Metadata}
\usage{
gguf_metadata(x)
}
\arguments{
\item{x}{A \code{gguf} object from \code{\link{gguf_load}}.}
}
\value{
A named list of metadata values.
}
\description{
Returns all key-value metadata pairs from a GGUF file as a named list.
}
