% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_type_size}
\alias{ggml_type_size}
\title{Get Type Size in Bytes}
\usage{
ggml_type_size(type)
}
\arguments{
\item{type}{GGML type constant (e.g., GGML_TYPE_F32)}
}
\value{
Size in bytes
}
\description{
Returns the size in bytes for all elements in a block for a given type.
}
