% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulkan.R
\name{ggml_vulkan_device_description}
\alias{ggml_vulkan_device_description}
\title{Get Vulkan device description}
\usage{
ggml_vulkan_device_description(device = 0L)
}
\arguments{
\item{device}{Device index (0-based)}
}
\value{
Character string with device description
}
\description{
Returns a human-readable description of the specified Vulkan device.
}
\examples{
\donttest{
if (ggml_vulkan_available() && ggml_vulkan_device_count() > 0) {
  ggml_vulkan_device_description(0)
}
}
}
