% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph.R
\name{ggml_gallocr_reserve}
\alias{ggml_gallocr_reserve}
\title{Reserve Memory for Graph}
\usage{
ggml_gallocr_reserve(galloc, graph)
}
\arguments{
\item{galloc}{Graph allocator object}

\item{graph}{Graph object}
}
\value{
TRUE on success, FALSE on failure
}
\description{
Pre-allocates memory for a graph. This is optional but recommended
when running the same graph multiple times to avoid reallocation.
}
