% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_blck_size}
\alias{ggml_blck_size}
\title{Get Block Size}
\usage{
ggml_blck_size(type)
}
\arguments{
\item{type}{GGML type constant}
}
\value{
Integer block size
}
\description{
Returns the block size for a GGML type. Quantized types process
data in blocks (e.g., 32 elements for Q4_0).
}
\examples{
ggml_blck_size(GGML_TYPE_F32)  # 1
ggml_blck_size(GGML_TYPE_Q4_0) # 32
}
\seealso{
Other type_system: 
\code{\link{ggml_ftype_to_ggml_type}()},
\code{\link{ggml_is_quantized}()},
\code{\link{ggml_type_name}()},
\code{\link{ggml_type_sizef}()}
}
\concept{type_system}
