% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{ggml_view_2d}
\alias{ggml_view_2d}
\title{2D View with Byte Offset (Graph)}
\usage{
ggml_view_2d(ctx, a, ne0, ne1, nb1, offset = 0)
}
\arguments{
\item{ctx}{GGML context}

\item{a}{Source tensor}

\item{ne0}{Size of dimension 0}

\item{ne1}{Size of dimension 1}

\item{nb1}{Stride for dimension 1 (in bytes)}

\item{offset}{Byte offset from the start of tensor data}
}
\value{
View tensor
}
\description{
Creates a 2D view of a tensor starting at a byte offset.
The view shares memory with the source tensor.
}
